spool 4-Funciones.log
prompt creando funciones de consulta...
CREATE OR REPLACE FUNCTION FLista_VotapPais (pcodpais IN PAISES.CODPAIS%TYPE)
                    RETURN VOTAPAIS_TAB IS
/******************************************************************************
 NOMBRE:      FLista_VotapPais
 DESCRIPCION: Dado un pas por parmetro: listar todas las votaciones que se han 
  producido en ese pas o censo global es decir, que han tenido asignado algn 
  censo que pertenezca aquel pas o censo global.

 PARAMETROS:
      INPUT:  pcodpais VARCHAR2(2): Codigo de Pais si no se informa se extraeran Todos
      OUTPUT: RSP
 RETORNA:
      Si es O.K. devuelve un Listado con las votaciones del pais o censo global
      Si no ha ido bien retornar un tipo de error:
             'ERROR: (SQLERRM)'
******************************************************************************/
   L_VOTAPAIS_TAB VOTAPAIS_TAB:= VOTAPAIS_TAB();
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);

   CURSOR Cur_Vota(p_codpais IN PAISES.CODPAIS%TYPE) IS
    SELECT vc.codpais,TTITULO, FFECINI, FFECFIN, NCAPVOT, NVOTTOT, DECODE(CESTADO,0,'En Edicion',1,'Pendiente',2,'Abierta',3,'Finalizada')testado, pparti, 
           decode(cestado,3,(select topcion from OPCIONES where idvota = v.idvota 
                                and nnumvot = (select max(nnumvot) from OPCIONES where idvota = v.idvota)),'-') Ganadora,
           decode(cestado,3,to_char((select prestot from OPCIONES where idvota = v.idvota 
                                        and nnumvot = (select max(nnumvot) from OPCIONES where idvota = v.idvota)),'999.99'),'-') prestot
      FROM VOTACION v, votacenso vc
     WHERE v.idvota = vc.idvota
       AND NVL(vc.codpais, NVL(p_codpais,'AA')) between nvl(p_codpais,'AA') AND nvl(p_codpais, 'ZZ')
    ORDER BY FFECINI;

  PRAGMA AUTONOMOUS_TRANSACTION;        
  
BEGIN
  --                
  FOR r IN Cur_Vota(pcodpais) LOOP
    L_VOTAPAIS_TAB.EXTEND;
    N := N + 1;
    L_VOTAPAIS_TAB(N) := VOTAPAIS_OBJ(r.codpais, r.ttitulo, r.ffecini, r.ffecfin, r.ncapvot, r.nvottot, r.testado, r.pparti, r.ganadora, r.prestot);            
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE,'FLista_VotapPais', 'CPAIS:'||nvl(pcodpais,' NULL - Todos los censos'), 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_VOTAPAIS_TAB;
  
EXCEPTION
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE, 'FLista_VotapPais', 'CPAIS:'||nvl(pcodpais,' NULL - Todos los censos'), 'RSP: '||RSP);

     COMMIT;
END FLista_VotapPais;
/
CREATE OR REPLACE FUNCTION FTopTenMasvota  RETURN VOTATOP_TAB IS
/******************************************************************************
 NOMBRE:      FTopTenMasvota
 DESCRIPCION: Muestra un listado con las 10 votaciones con ms diferencia 
  porcentual de votos entre la opcin mas y menos votadas.


 PARAMETROS:
      INPUT:  NINGUNO
      OUTPUT: RSP
 RETORNA:
      Si ha ido bien devuelve un Listado de las 10 votaciones con MAS diferencia
      porcentual.
      Si no ha ido bien retorna el siguiente error:
              'ERROR: (SQLERRM)' 
******************************************************************************/

   L_VOTATOP_TAB VOTATOP_TAB:= VOTATOP_TAB();
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);

   CURSOR Cur_Vota IS
       select TTITULO, FFECINI, FFECFIN, NCAPVOT, NVOTTOT, pparti, masvotada, prestotmax, menosvotada, prestotmin, 
              (prestotmax - prestotmin)diferen
         from (SELECT TTITULO, FFECINI, FFECFIN, NCAPVOT, NVOTTOT, pparti, 
                 (select topcion from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select max(nnumvot) from OPCIONES where idvota = p.idvota)) MasVotada,
                 (select prestot from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select max(nnumvot) from OPCIONES where idvota = p.idvota)) PrestotMax,
                 (select topcion from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select min(nnumvot) from OPCIONES where idvota = p.idvota)) MenosVotada,
                 (select prestot from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select min(nnumvot) from OPCIONES where idvota = p.idvota)) PrestotMin
                FROM VOTACION p
               WHERE cestado=3) a
          where ROWNUM   <= 10                 
       order by  diferen desc;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
                
  FOR r IN Cur_Vota LOOP
    L_VOTATOP_TAB.EXTEND;
    N := N + 1;
    L_VOTATOP_TAB(N) := VOTATOP_OBJ(r.ttitulo, r.ffecini, r.ffecfin, r.ncapvot, r.nvottot, r.pparti, 
                                    r.masvotada, r.prestotmax, r.menosvotada, r.prestotmin, r.diferen); 
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
       VALUES(SYSDATE,'FTopTenMasvota', 'No se requieren', 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_VOTATOP_TAB;

  
EXCEPTION
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
        VALUES(SYSDATE, 'FTopTenMasvota', 'No se requieren', 'RSP: '||RSP);

     COMMIT;
     
  
END FTopTenMasvota;
/
CREATE OR REPLACE FUNCTION FTopTenMenvota RETURN VOTATOP_TAB IS
/******************************************************************************
 NOMBRE:      FTopTenMenvota
 DESCRIPCION: Muestra un listado con las 10 votaciones con menos diferencia 
  porcentual de votos entre la opcin mas y menos votadas.


 PARAMETROS:
      INPUT:  NINGUNO
      OUTPUT: RSP
 RETORNA:
      Si ha ido bien devuelve un Listado de las 10 votaciones con MENOS diferencia
      porcentual.
      Si no ha ido bien retorna el siguiente error:
             'ERROR: (SQLERRM)'  
******************************************************************************/

   L_VOTATOP_TAB VOTATOP_TAB:= VOTATOP_TAB();
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);

   CURSOR Cur_Vota IS
       select TTITULO, FFECINI, FFECFIN, NCAPVOT, NVOTTOT, pparti, masvotada, prestotmax, menosvotada, prestotmin, 
              (prestotmax - prestotmin)diferen
         from (SELECT TTITULO, FFECINI, FFECFIN, NCAPVOT, NVOTTOT, pparti, 
                 (select topcion from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select max(nnumvot) from OPCIONES where idvota = p.idvota)) MasVotada,
                 (select prestot from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select max(nnumvot) from OPCIONES where idvota = p.idvota)) PrestotMax,
                 (select topcion from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select min(nnumvot) from OPCIONES where idvota = p.idvota)) MenosVotada,
                 (select prestot from OPCIONES where idvota = p.idvota 
                  and nnumvot = (select min(nnumvot) from OPCIONES where idvota = p.idvota)) PrestotMin
                FROM VOTACION p
               WHERE cestado=3) a
          where ROWNUM   <= 10                 
       order by  diferen asc;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN
                
  FOR r IN Cur_Vota LOOP
    L_VOTATOP_TAB.EXTEND;
    N := N + 1;
    L_VOTATOP_TAB(N) := VOTATOP_OBJ(r.ttitulo, r.ffecini, r.ffecfin, r.ncapvot, r.nvottot, r.pparti, 
                                    r.masvotada, r.prestotmax, r.menosvotada, r.prestotmin, r.diferen);            
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
       VALUES(SYSDATE,'FTopTenMenvota','No se requieren', 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_VOTATOP_TAB;

  
EXCEPTION
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
        VALUES(SYSDATE, 'FTopTenMenvota', 'No se requieren', 'RSP: '||RSP);

     COMMIT;
     
  
END FTopTenMenvota;
/
CREATE OR REPLACE FUNCTION FcensoPersona (pidperson IN PERSONAS.IDPERSON%TYPE)
RETURN PERSOCENSO_TAB IS
/******************************************************************************
 NOMBRE:      FcensoPersona
 DESCRIPCION: Dado el identificador de persona por parmetro: lista todos los 
  censos a los que ha pertenecido o pertenece.


 PARAMETROS:
      INPUT:  idperson NUMBER(8): Identificador de la persona. Obligatorio
      OUTPUT: RSP
 RETORNA:
      Si ha ido bien devuelve un Listado de los censos de la persona
      Si no ha ido bien retorna el siguiente error:
              'ERROR: Tipo de Error' 
******************************************************************************/
   L_PERSOCENSO_TAB PERSOCENSO_TAB:= PERSOCENSO_TAB();
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   par_in            VARCHAR2(500);
   E_ERROR EXCEPTION;

   CURSOR Cur_Perso(p_idperson IN PERSONAS.IDPERSON%TYPE) IS
        SELECT ps.idperson, ps.fnacimi, ps.csexper, pc.falta, pc.fbaja, p.codpais, p.tpais, r.tregion, l.tlocali
          FROM personas ps, PERSOCENSO pc, paises p, region r, localidad l
         WHERE ps.idperson = pc.idperson
           AND pc.codpais = p.codpais
          AND pc.codpais = r.codpais
          AND pc.idregion = r.idregion
          AND pc.codpais = l.codpais
          AND pc.idregion = l.idregion
          AND pc.idlocali = l.idlocali
          AND ps.idperson = p_idperson
        ORDER BY ps.idperson, pc.falta;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  par_in := 'pidperson: ' ||pidperson;
  
  --Verifica que el campo obligatorio tenga valor
  RSP := 'ERROR: La persona es Obligatoria, ejecutar FcensoPersona(IDSPERSON)';
  IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;
                
  FOR r IN Cur_Perso(pidperson) LOOP
    L_PERSOCENSO_TAB.EXTEND;
    N := N + 1;
    L_PERSOCENSO_TAB(N) := PERSOCENSO_OBJ(r.idperson, r.fnacimi, r.csexper, r.falta, r.fbaja, r.codpais, r.tpais, r.tregion, r.tlocali);            
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE,'FcensoPersona', par_in, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_PERSOCENSO_TAB;

  
EXCEPTION
  WHEN E_ERROR THEN
     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE, 'FcensoPersona', par_in, 'RSP: '||RSP);
     COMMIT;
     
     RETURN NULL;
     
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE, 'FcensoPersona', par_in, 'RSP: '||RSP);

     COMMIT;
     RETURN NULL;
  
END FcensoPersona;
/
CREATE OR REPLACE FUNCTION FVOTAPERSONA (pidperson IN PERSONAS.IDPERSON%TYPE)
RETURN PERSOVOTA_TAB IS
/******************************************************************************
 NOMBRE:      FVOTAPERSONA
 DESCRIPCION: Dado el identificador de persona por parmetro: lista todas las 
  votaciones en las que ha participado y las que ha podido participar y no lo 
  ha echo.


 PARAMETROS:
      INPUT:  idperson NUMBER(8): Identificador de la persona. Obligatorio
      OUTPUT: RSP
 RETORNA:
      Si ha ido bien devuelve un Listado de todas las votaciones que ha participado 
      y las que no.
      Si no ha ido bien retorna el siguiente error:
              'ERROR: Tipo de Error' 
******************************************************************************/
   L_PERSOVOTA_TAB PERSOVOTA_TAB:= PERSOVOTA_TAB();
   N                 NUMBER := 0;
   RSP               VARCHAR2(2000);
   err_num           NUMBER;
   err_msg           VARCHAR2(100);
   par_in            VARCHAR2(500);
   E_ERROR EXCEPTION;

   CURSOR Cur_Perso(p_idperson IN PERSONAS.IDPERSON%TYPE) IS
        SELECT p.idperson, p.fnacimi, p.csexper, pc.falta, pc.fbaja, v.ttitulo, v.ffecini, v.ffecfin,
               (DECODE(GREATEST((SELECT count(1) 
                                   FROM VOTAPERSONA
                                  WHERE idvota = vc.idvota 
                                    AND idperson = p.idperson),0), 0, 'No','Si'))Participo
          FROM PERSONAS p, PERSOCENSO pc, VOTACENSO vc, VOTACION v
         WHERE p.idperson = pc.idperson
           AND pc.codpais = nvl(vc.codpais,pc.codpais)
           AND pc.idregion = nvl(vc.idregion,pc.idregion)
           AND pc.idlocali = nvl(vc.idlocali,pc.idlocali)
           AND v.ffecini BETWEEN pc.falta AND NVL(pc.fbaja,to_date('31122999','DDMMYYYY'))
           AND vc.idvota = v.idvota
           AND p.idperson = 1
         order by p.idperson;
        
  PRAGMA AUTONOMOUS_TRANSACTION;
    
BEGIN

  par_in := 'pidperson: ' ||pidperson;
  
  --Verifica que el campo obligatorio tenga valor
  RSP := 'ERROR: La persona es Obligatoria, ejecutar FcensoPersona(IDSPERSON)';
  IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;
                
  FOR r IN Cur_Perso(pidperson) LOOP
    L_PERSOVOTA_TAB.EXTEND;
    N := N + 1;
    L_PERSOVOTA_TAB(N) := PERSOVOTA_OBJ(r.idperson, r.fnacimi, r.csexper, r.falta, r.fbaja, r.ttitulo,
                                        r.ffecini, r.ffecfin, r.participo);            
  END LOOP;
  
  RSP := 'OK';  

  INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE,'FVOTAPERSONA', par_in, 'RSP: '||RSP);
  
  COMMIT;
  
  RETURN L_PERSOVOTA_TAB;

  
EXCEPTION
  WHEN E_ERROR THEN
     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE, 'FVOTAPERSONA', par_in, 'RSP: '||RSP);
     COMMIT;
     
     RETURN NULL;
     
  WHEN OTHERS THEN
     err_num := SQLCODE;
     err_msg := SUBSTR(SQLERRM, 1, 200);

     RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;
     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT) VALUES(SYSDATE, 'FVOTAPERSONA', par_in, 'RSP: '||RSP);

     COMMIT;
     RETURN NULL;
  
END FVOTAPERSONA;
/
spool off;