spool 5-pk_censos.log
prompt creando packages pk_censos...
CREATE OR REPLACE PACKAGE PK_CENSOS AS
/******************************************************************************
   NOMBRE:   PK_CENSOS
   DESCRIPCIN: GESTION DE TODA LA INFORMACIN RELACIONADA CON LOS CENSOS.

******************************************************************************/


  /**************************************
   *  MANTENIMIENTO PAISES
   **************************************/
  PROCEDURE INS_PAIS (pcodpais IN PAISES.CODPAIS%TYPE,
                      ptpais   IN PAISES.TPAIS%TYPE,
                      RSP      OUT VARCHAR2);

  PROCEDURE DEL_PAIS (pcodpais IN PAISES.CODPAIS%TYPE,
                      RSP      OUT VARCHAR2);

  PROCEDURE UPD_PAIS (pcodpais IN PAISES.CODPAIS%TYPE,
                      ptpais   IN PAISES.TPAIS%TYPE,
                      RSP      OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO REGIONES
   **************************************/
  PROCEDURE INS_REGION (pcodpais IN REGION.CODPAIS%TYPE,
                        ptregion IN REGION.TREGION%TYPE,
                        RSP      OUT VARCHAR2);

  PROCEDURE DEL_REGION (pidregion IN REGION.IDREGION%TYPE,
                        RSP       OUT VARCHAR2);

  PROCEDURE UPD_REGION (pidregion IN REGION.IDREGION%TYPE,
                        ptregion  IN REGION.TREGION%TYPE,
                        RSP       OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO LOCALIDAD
   **************************************/
  PROCEDURE INS_LOCALI (pcodpais  IN LOCALIDAD.CODPAIS%TYPE,
                        pidregion IN LOCALIDAD.IDREGION%TYPE,
                        ptlocali  IN LOCALIDAD.TLOCALI%TYPE,
                        RSP      OUT VARCHAR2);

  PROCEDURE DEL_LOCALI (pidlocali IN LOCALIDAD.IDLOCALI%TYPE,
                        RSP       OUT VARCHAR2);

  PROCEDURE UPD_LOCALI (pidlocali IN LOCALIDAD.IDLOCALI%TYPE,
                        ptlocali  IN LOCALIDAD.TLOCALI%TYPE,
                        RSP       OUT VARCHAR2);

END PK_CENSOS;
/
CREATE OR REPLACE PACKAGE BODY PK_CENSOS AS

  /******************************************************************************
   NOMBRE:      INS_PAIS
   DESCRIPCIN: Da de alta un nuevo Pas, si:
                  -Todos los campos estan informados
                  -El registro no EXISTA
   PARAMETROS:
        INPUT:  Obligatorios: pcodpais. VARCHAR2(02): Codigo del Pas
                                ptpais. VARCHAR2(30): Nombre del Pas

       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No existe el pas a crear
   POSTCONDICION: Se da de alta el pas
   COMENTARIOS: Los parmetros de entrada se convertirn en maysculas y sin espacios.
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: El Cdigo del Pas no puede ser nulo'
                'ERROR: La Descripcin del Pas no puede ser nula'
                'ERROR: El Pas ya ha sido dado de alta'
                'ERROR: (sqlerrm)'

   ******************************************************************************/
   PROCEDURE INS_PAIS (pcodpais     IN PAISES.CODPAIS%TYPE,
                       ptpais       IN PAISES.TPAIS%TYPE,
                       RSP          OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lcodpais      PAISES.CODPAIS%TYPE;
   ltpais        PAISES.TPAIS%TYPE;


   BEGIN
     lcodpais := UPPER(TRIM(pcodpais));
     ltpais := UPPER(TRIM(ptpais));

     PAR_IN := 'pcodpais: '||lcodpais||', ptpais: '||ltpais;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Cdigo del Pas no puede ser nulo';
     IF lcodpais IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin del Pas no puede ser nula';
     IF ltpais IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que el registro no se haya insertado previamente por Cdigo o
     --Descripcin
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PAISES
        WHERE (codpais  = pcodpais
           OR TPAIS LIKE  '%'||ltpais||'%')
          AND ROWNUM  = 1;
        RSP := 'ERROR: El Pas ya ha sido dado de alta';
        RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;
     -- Insercin en la tabla
     INSERT INTO PAISES (codpais, tpais)
      VALUES (lcodpais, ltpais);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
          VALUES(SYSDATE,'PK_CENSOS.INS_PAIS', PAR_IN, 'RSP: '||RSP);

     COMMIT;
     --
  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_PAIS', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_PAIS', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_PAIS;

  /******************************************************************************
   NOMBRE:      DEL_PAIS
   PROPSITO:   Da de baja un Pas, si previamente supera los siguientes controles:
                 -El campo codpais este informado
                 -El Pas exista en la Base de datos
                 -El Pas no tenga dependencias
                 -Si tiene dependencias se realizar una baja lgica en cascada 
                 informando la fecha de baja en todas sus dependencias, sino se
                 realizar la baja fsica.

   PARAMETROS:
        INPUT:  Obligatorios: pcodpais VARCHAR2(02): Cdigo del Pas
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista el pas a dar de baja
   POSTCONDICION: Se da de baja el pas
   COMENTARIOS: Los parmetros de entrada se convertirn en maysculas y sin espacios.
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Cdigo del Pas no puede ser nulo'
                'ERROR: El Pas no existe'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE DEL_PAIS (pcodpais  IN PAISES.CODPAIS%TYPE,
                      RSP      OUT VARCHAR2) IS
   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcodpais      PAISES.CODPAIS%TYPE;


   BEGIN

     lcodpais := UPPER(TRIM(pcodpais));
     PAR_IN := 'pcodpais: '||lcodpais;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El Cdigo del Pas no puede ser nulo';
     IF lcodpais IS NULL THEN  RAISE E_ERROR; END IF;


     --Verifica que el Pas exista en la Base de datos
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PAISES
        WHERE CODPAIS = lcodpais
          and fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El Pas no existe o est dado de baja';
           RAISE E_ERROR;
     END;

     --Si se pasa las pruebas de validacin realiza el borrado Fsico/Lgico del Pas
     BEGIN
        DELETE PAISES WHERE  CODPAIS = lcodpais;
     EXCEPTION
       WHEN OTHERS THEN
            BEGIN
            	UPDATE LOCALIDAD SET fbaja = TRUNC(SYSDATE)
            	 WHERE codpais = lcodpais;
            	--
            	UPDATE REGION SET fbaja = TRUNC(SYSDATE)
            	 WHERE codpais = lcodpais;
            	--
            	UPDATE PAISES SET fbaja = TRUNC(SYSDATE)
            	 WHERE codpais = lcodpais;
            EXCEPTION
             WHEN OTHERS THEN
                  RSP := SUBSTR(SQLERRM,1,200);
                  RAISE E_ERROR;
            END;
     END;
     --
     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.DEL_PAIS', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_PAIS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_PAIS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_PAIS;

  /******************************************************************************
   NOMBRE:      UPD_PAIS
   DESCRIPCION: Realiza una modificacin en la descripcin de un Pas. Si:
   							- El registro existe
   							- La nueva descripcin viene informada

   PARAMETROS:
        INPUT:  Obligatorios: pcodpais VARCHAR2(02): Cdigo del pas
                              ptpais VARCHAR2(30): Descripcin del pas
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Existe el pas a modificar
   POSTCONDICION: Se modifica la descripcin del pas
   COMENTARIOS: Los parmetros de entrada se convertirn en maysculas y sin espacios.
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Cdigo del pas no puede ser nulo'
                'ERROR: La Descripcin del pas no puede ser nulo'
                'ERROR: El pas no existe o est dado de baja'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE UPD_PAIS (pcodpais IN PAISES.CODPAIS%TYPE,
                      ptpais   IN PAISES.TPAIS%TYPE,
                      RSP      OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcodpais      PAISES.CODPAIS%TYPE;
   ltpais        PAISES.TPAIS%TYPE;

   BEGIN
     lcodpais := UPPER(TRIM(pcodpais));
     ltpais := UPPER(TRIM(ptpais));

     PAR_IN := 'pcodpais: '||lcodpais||', ptpais: '||ltpais;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Cdigo del Pas no puede ser nulo';
     IF lcodpais IS NULL  THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: La Descripcin del Pas no puede ser nula';
     IF ltpais IS NULL  THEN  RAISE E_ERROR; END IF;


     --Verifica que el Pais exista en la Base de datos
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM PAISES
        WHERE codpais = lcodpais
          AND fbaja IS NULL;

      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El Pas no existe o est dado de baja';
           RAISE E_ERROR;
      END;


     --Si se pasan las pruebas de validacin actualiza
     UPDATE PAISES  SET  tpais = ltpais
     WHERE   codpais  = lcodpais;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.UPD_PAIS', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_PAIS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_PAIS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_PAIS;

  /******************************************************************************
   NOMBRE:      INS_REGION
   DESCRIPCIN: Da de alta una Region a un pas, si cumple que:
                  -Todos los campos esten informados
                  -El pas exista
                  -El registro no EXISTA
   PARAMETROS:
        INPUT:  Obligatorios: pcodpais VARCHAR2(02): Cdigo del Pas
                              ptregion VARCHAR2(50): Nombre de la Regin

       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No existe la regin del pais a crear
   POSTCONDICION: Se da de alta la regin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: El Cdigo del Pas no puede ser nulo'
                'ERROR: La Regin no puede ser nula'
                'ERROR: El pas no existe o est dado de baja'
                'ERROR: La Regin ya ha sido dado de alta para el pas'
                'ERROR: (sqlerrm)'

  ******************************************************************************/
  PROCEDURE INS_REGION (pcodpais     IN REGION.CODPAIS%TYPE,
                        ptregion     IN REGION.TREGION%TYPE,
                        RSP          OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lcodpais      REGION.CODPAIS%TYPE;
   ltregion      REGION.TREGION%TYPE;


   BEGIN

     lcodpais := TRIM(UPPER(pcodpais));
     ltregion := TRIM(UPPER(ptregion));

     PAR_IN := 'pcodpais: '||lcodpais||', ptregion: '||ltregion;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Cdigo del Pas no puede ser nulo';
     IF lcodpais IS NULL THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: La Descripcin de la Regin no puede ser nula';
     IF ltregion IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que el pais exista
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PAISES
        WHERE codpais  = lcodpais
          AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: El pas no existe o est dado de baja';
            RAISE E_ERROR;
     END;

     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM REGION
        WHERE codpais  = lcodpais
          AND tregion  = ltregion
          AND ROWNUM  = 1;
        RSP := 'ERROR: la regin ya ha sido dado de alta para el pas';
        RAISE E_ERROR;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
             NULL;
     END;

     -- Insercin en la tabla
     INSERT INTO REGION (codpais, idregion, tregion) VALUES (lcodpais, seq_region.nextval, ltregion);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.INS_REGION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_REGION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_REGION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_REGION;

  /******************************************************************************
   NOMBRE:      DEL_REGION
   PROPSITO:   Da de baja una regin de un Pas, si:
                 -El campo idregion esta informado
                 -La regin exista en la Base de datos
                 -Si tiene dependencias se realizar una baja lgica en cascada 
                 informando la fecha de baja en todas sus dependencias, sino se
                 realizar la baja fsica.
                 
   PARAMETROS:
        INPUT:  Obligatorios:
                     pidregion NUMBER(08): Identificador de Regin
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la regin a dar de baja
   POSTCONDICION: Se da de baja la regin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Identificador de Regin no puede ser nulo'
                'ERROR: La Regin no existe o est dada de baja'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE DEL_REGION (pidregion IN REGION.IDREGION%TYPE,
                        RSP      OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);


   BEGIN

     PAR_IN := 'pidregion:'||pidregion;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El Identificador de la regin no puede ser nulo';
     IF pidregion IS NULL THEN  RAISE E_ERROR; END IF;
     --Verifica que la Regin exista en la Base de datos
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM REGION
        WHERE idregion = pidregion
          AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La regin no existe o est dada de baja';
           RAISE E_ERROR;
     END;
     --Si se pasa las pruebas de valicacin realiza el borrado Fisico/Lgico de
     -- la Regin
     BEGIN
        DELETE REGION WHERE  idregion = pidregion;
     EXCEPTION
       WHEN OTHERS THEN
            BEGIN
            	UPDATE LOCALIDAD SET fbaja = TRUNC(SYSDATE)
            	 WHERE idregion = pidregion;
            	--
            	UPDATE REGION SET fbaja = TRUNC(SYSDATE)
            	 WHERE idregion = pidregion;
            	--
            EXCEPTION
             WHEN OTHERS THEN
                  RSP := SUBSTR(SQLERRM,1,200);
                  RAISE E_ERROR;
            END;
     END;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
          VALUES(SYSDATE,'PK_CENSOS.DEL_REGION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_REGION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_REGION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_REGION;

  /******************************************************************************
   NOMBRE:      UPD_REGION
   DESCRIPCION: Realiza una modificacin en la descripcin de una Regin.

   PARAMETROS:
        INPUT:  Obligatorios: pidregion NUMBER(08): identificador de la regin
                               ptregion VARCHAR2(50): Descripcin de la regin
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION:  Exista la regin a modificar
   POSTCONDICION: Se modifica la descripcin de la regin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Identificador de la Regin no puede ser nulo'
                'ERROR: La Descripcin de la Regin no puede ser nulo'
                'ERROR: La Regin no existe o est dada de baja'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE UPD_REGION (pidregion IN REGION.IDREGION%TYPE,
                        ptregion  IN REGION.TREGION%TYPE,
                        RSP       OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   ltregion      REGION.TREGION%TYPE;

   BEGIN

     ltregion := UPPER(TRIM(ptregion));
     PAR_IN := 'pidregion:'||pidregion||', ptregion: '||ltregion;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Identificador de la Regin no puede ser nulo';
     IF pidregion IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin de la Regin no puede ser nula';
     IF ltregion IS NULL  THEN  RAISE E_ERROR; END IF;


     --Verifica que la region exista en la Base de datos
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM REGION
        WHERE idregion = pidregion
          AND fbaja IS NULL;

     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La Regin no existe o est dada de baja';
           RAISE E_ERROR;
     END;


     --Si se pasan las pruebas de validacin actualiza
     UPDATE REGION
        SET tregion  = ltregion
     WHERE  idregion = pidregion;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.UPD_REGION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_REGION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_REGION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_REGION;

  /******************************************************************************
   NOMBRE:      INS_LOCALI
   DESCRIPCIN: Da de alta una Localidad que forma parte de una Regin/Pas, si cumple que:
                  -Todos los campos esten informados
                  -El pas exista
                  -La regin exista
                  -El registro no EXISTA
   PARAMETROS:
        INPUT:  Obligatorios: pcodpais  VARCHAR2(02): Codigo del Pas
                              pidregion NUMBER(08): Identificador de la Regin
                              ptlocali  VARCHAR2(50): Nombre de la Localidad

       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No existe la localidad para la regin/pas
   POSTCONDICION: Se da de alta la localidad
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: El Cdigo del Pas no puede ser nulo'
                'ERROR: La Regin no puede ser nula'
                'ERROR: La descripcin de la localidad no puede ser nula'
                'ERROR: El pas no existe o est dado de baja'
                'ERROR: La regin/pas no existe o est dada de baja'
                'ERROR: la localidad ya ha sido dada de alta para esta regin/pas'
                'ERROR: (sqlerrm)'

  ******************************************************************************/
  PROCEDURE INS_LOCALI (pcodpais  IN LOCALIDAD.CODPAIS%TYPE,
                        pidregion IN LOCALIDAD.IDREGION%TYPE,
                        ptlocali  IN LOCALIDAD.TLOCALI%TYPE,
                        RSP      OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lcodpais      LOCALIDAD.CODPAIS%TYPE;
   ltlocali      LOCALIDAD.TLOCALI%TYPE;


   BEGIN
     --
     lcodpais := TRIM(UPPER(pcodpais));
     ltlocali := TRIM(UPPER(ptlocali));
     PAR_IN := 'pcodpais: '||lcodpais||', pidregion: '||pidregion||', ptlocali: '||ltlocali;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Cdigo del Pas no puede ser nulo';
     IF lcodpais IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El Identificacin de la Regin no puede ser nulo';
     IF pidregion IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin de la localidad no puede ser nula';
     IF ltlocali IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que el pais exista
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PAISES
        WHERE codpais  = lcodpais
          AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: El pas no existe o est dado de baja';
            RAISE E_ERROR;
     END;
     --Verifica que la regin/Pais exista
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM region
        WHERE codpais  = lcodpais
          AND idregion = pidregion
          AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La regin/pas no existe o est dado de baja';
            RAISE E_ERROR;
     END;
     --Verifica que el registro no se haya insertado previmente con otro ID
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM LOCALIDAD
        WHERE codpais  = lcodpais
          AND idregion = pidregion
          AND tlocali  = ltlocali
          AND ROWNUM  = 1;
       RSP := 'ERROR: la localidad ya ha sido dada de alta';
       RAISE E_ERROR;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
             NULL;
     END;

     -- Insercin en la tabla
     INSERT INTO LOCALIDAD (codpais, idregion, idlocali, tlocali)
                   VALUES (lcodpais, pidregion, seq_locali.nextval, ltlocali);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.INS_LOCALI', PAR_IN, 'RSP: '||RSP);

     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_LOCALI', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_CENSOS.INS_LOCALI', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_LOCALI;

  /******************************************************************************
   NOMBRE:      DEL_LOCALI
   DESCRIPCIN:   Da de baja una localidad, si:
                 -El campo idlocali esta informado
                 -La localidad existe en la Base de datos
                 -Si tiene dependencias se realizar una baja lgica en cascada 
                 informando la fecha de baja en todas sus dependencias, sino se
                 realizar la baja fsica.
                 
   PARAMETROS:
        INPUT:  Obligatorios:
                     pidlocali NUMBER(08): Identificador de la Localidad
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la localidad a dar de baja
   POSTCONDICION: Se da de baja la localidad
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Identificador de localidad no puede ser nulo'
                'ERROR: La localidad no existe o est dada de baja'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE DEL_LOCALI (pidlocali IN LOCALIDAD.IDLOCALI%TYPE,
                        RSP      OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);


   BEGIN

     PAR_IN := 'pidlocali:'||pidlocali;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El Identificador de la localidad no puede ser nulo';
     IF pidlocali IS NULL THEN  RAISE E_ERROR; END IF;
     --Verifica que la localidad exista en la Base de datos
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM localidad
        WHERE idlocali = pidlocali
         AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La localidad no existe o est dada de baja';
           RAISE E_ERROR;
     END;
     --Si se pasa las pruebas de valicacin realiza el borrado Fisico/Lgico de
     -- la Regin
     BEGIN
        DELETE LOCALIDAD WHERE  idlocali = pidlocali;
     EXCEPTION
       WHEN OTHERS THEN
            BEGIN
            	UPDATE LOCALIDAD SET fbaja = TRUNC(SYSDATE)
            	 WHERE idlocali = pidlocali;
            EXCEPTION
             WHEN OTHERS THEN
                  RSP := SUBSTR(SQLERRM,1,200);
                  RAISE E_ERROR;
            END;
     END;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
          VALUES(SYSDATE,'PK_CENSOS.DEL_LOCALI', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_LOCALI', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.DEL_LOCALI', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_LOCALI;

  /******************************************************************************
   NOMBRE:      UPD_LOCALI
   DESCRIPCION: Realiza una modificacin en la descripcin de una Localidad.

   PARAMETROS:
        INPUT:  Obligatorios: pidlocali NUMBER(08): identificador de la localidad
                              ptlocali  VARCHAR2(50): Descripcin de la localidad
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION:  Exista la localidad a modificar
   POSTCONDICION: Se modifica la descripcin de la localidad
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El Identificador de la localidad no puede ser nulo'
                'ERROR: La Descripcin de la localidad no puede ser nulo'
                'ERROR: La localidad no existe o est dada de baja'
                'ERROR: (sqlerrm)'
  ******************************************************************************/
  PROCEDURE UPD_LOCALI (pidlocali IN LOCALIDAD.IDLOCALI%TYPE,
                        ptlocali  IN LOCALIDAD.TLOCALI%TYPE,
                        RSP       OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   ltlocali      LOCALIDAD.TLOCALI%TYPE;

   BEGIN
     ltlocali := UPPER(TRIM(ptlocali));
     PAR_IN := 'pidlocali:'||pidlocali||', ptlocali: '||ltlocali;

     --Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Identificador de la Localidad no puede ser nulo';
     IF pidlocali IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin de la Localidad no puede ser nula';
     IF ltlocali IS NULL  THEN  RAISE E_ERROR; END IF;


     --Verifica que la localidad exista en la Base de datos
     BEGIN

       SELECT 'S'
         INTO EXISTE
         FROM LOCALIDAD
        WHERE idlocali = pidlocali
          AND fbaja IS NULL;

     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La localidad no existe o est dada de baja';
           RAISE E_ERROR;
     END;


     --Si se pasan las pruebas de validacin actualiza
     UPDATE LOCALIDAD
        SET tlocali  = ltlocali
     WHERE  idlocali = pidlocali;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_CENSOS.UPD_LOCALI', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_LOCALI', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_CENSOS.UPD_LOCALI', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_LOCALI;
  --
END PK_CENSOS;
/
spool off;