spool 6-pk_personas.log
prompt creando packages pk_personas...
CREATE OR REPLACE PACKAGE PK_PERSONAS AS
/******************************************************************************
   NOMBRE:   PK_PERSONAS
   DESCRIPCIN: GESTION DE TODA LA INFORMACIN RELACIONADA CON LAS PERSONAS.

******************************************************************************/


  /**************************************
   *  MANTENIMIENTO PERSONAS
   **************************************/
  PROCEDURE INS_PERSONAS (pfnacimi  IN PERSONAS.FNACIMI%TYPE,
                          pcsexper  IN PERSONAS.CSEXPER%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE DEL_PERSONAS (pidperson IN PERSONAS.IDPERSON%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE UPD_PERSONAS (pidperson IN PERSONAS.IDPERSON%TYPE,
                          pfnacimi  IN PERSONAS.FNACIMI%TYPE,
                          pcsexper  IN PERSONAS.CSEXPER%TYPE,
                          RSP       OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO CENSO - PERSONAS
   **************************************/
  PROCEDURE INS_PERSOCENSO (pidperson IN PERSOCENSO.IDPERSON%TYPE,
                            pcodpais  IN PERSOCENSO.CODPAIS%TYPE,
                            pidregion IN PERSOCENSO.IDREGION%TYPE,
                            pidlocali IN PERSOCENSO.IDLOCALI%TYPE,
                            RSP      OUT VARCHAR2);

  PROCEDURE DEL_PERSOCENSO(pidperson    IN PERSOCENSO.IDPERSON%TYPE,
                           RSP          OUT VARCHAR2);

  /**************************************
   *  VOTACION PERSONAS
   **************************************/
  PROCEDURE INS_VOTAPERSONA (pidvota   IN VOTAPERSONA.IDVOTA%TYPE,
                             pidperson IN VOTAPERSONA.IDPERSON%TYPE,
                             pnopcio   IN VOTAPERSONA.NOPCIO%TYPE,
                             RSP      OUT VARCHAR2);
END PK_PERSONAS;
/
CREATE OR REPLACE PACKAGE BODY PK_PERSONAS AS

  /******************************************************************************
   NOMBRE:      INS_PERSONAS
   DESCRIPCIN: Da de alta una persona, si:
                  -Todos los campos obligatorios estn informados
                  -y las validaciones sean correctas
   PARAMETROS:
        INPUT:  Obligatorios: pfnacimi DATE: Fecha de nacimiento
                              pcsexper NUMBER(1): Sexo
       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Los campos obligatorios estn informados
   POSTCONDICION: Se da de alta la persona
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: La fecha de nacimiento no puede ser nulo'
                'ERROR: El sexo no puede ser nulo'
                'ERROR: El valor del sexo es incorrecto. Valores correctos son 1-hombre, 2-mujer'
                'ERROR: (SQLERRM)'

   ******************************************************************************/
  PROCEDURE INS_PERSONAS (pfnacimi  IN PERSONAS.FNACIMI%TYPE,
                          pcsexper  IN PERSONAS.CSEXPER%TYPE,
                          RSP       OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;


   BEGIN
     PAR_IN := 'pfnacimi: '||pfnacimi||', pcsexper: '||pcsexper;

     --Verifica los parametros de entrada
     RSP := 'ERROR: La fecha de nacimiento no puede ser nulo';
     IF pfnacimi IS NULL THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: El sexo no puede ser nulo';
     IF pcsexper IS NULL  THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: El valor del sexo es incorrecto. Valores correctos son 1-hombre, 2-mujer' ;
     IF pcsexper not in (1,2)  THEN  RAISE E_ERROR; END IF;


     -- Pasa las validaciones e Inserta en la tabla
     INSERT INTO PERSONAS (IDPERSON, FNACIMI, CSEXPER)
                  VALUES (seq_idperson.nextval, pfnacimi, pcsexper);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.INS_PERSONAS', PAR_IN, 'RSP: '||RSP);

     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_PERSONAS.INS_PERSONAS', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_PERSONAS.INS_PERSONAS', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_PERSONAS;

  /******************************************************************************
   NOMBRE:      DEL_PERSONAS
   PROPSITO:   Da de baja una persona, si previamente supera los siguientes controles:
                 -El campo idperson este informado
                 -La persona exista en la Base de datos

   PARAMETROS:
        INPUT:  Obligatorios: pidperson NUMBER(8): Identificador de la persona
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la persona
   POSTCONDICION: Se da de baja lgica la persona, para mantener el histrico
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El identificacin de la persona no puede ser nulo'
                'ERROR: La persona no existe'
                'ERROR: (SQLERRM)'
   ******************************************************************************/
  PROCEDURE DEL_PERSONAS (pidperson IN PERSONAS.IDPERSON%TYPE,
                          RSP       OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN

     PAR_IN := 'pidperson: '||pidperson;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El identificacin de la persona no puede ser nulo';
     IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la persona exista en la Base de datos
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PERSONAS
        WHERE IDPERSON = pidperson;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona no existe';
           RAISE E_ERROR;
      END;

      --Si se pasa las pruebas de valicacin realiza el borrado Fisico de la persona
      BEGIN
        DELETE PERSONAS WHERE IDPERSON = pidperson;
      EXCEPTION
       WHEN OTHERS THEN
            BEGIN
            	UPDATE PERSONAS SET  FBAJA = TRUNC(SYSDATE)
            	  WHERE idperson = pidperson;
            	UPDATE PERSOCENSO SET FBAJA = TRUNC(SYSDATE)
            	  WHERE idperson = pidperson;
            EXCEPTION
              WHEN OTHERS THEN
                   RSP := SUBSTR(SQLERRM,1,200);
                   RAISE E_ERROR;
            END;
      END;

      RSP := 'OK';

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.DEL_PERSONAS', PAR_IN, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.DEL_PERSONAS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.DEL_PERSONAS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_PERSONAS;

  /******************************************************************************
   NOMBRE:      UPD_PERSONAS
   DESCRIPCION: Realiza la modificacin de los datos de una persona.

   PARAMETROS:
        INPUT:  Obligatorios: pidperson NUMBER(8): Identificador de la persona
                              pfnacimi  DATE:  Fecha de nacimiento
                              psexper   NUMBER(1):   Sexo

        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la persona a modificar
   POSTCONDICION: Se realiza la modificacin de la persona
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la persona no puede ser nulo'
                'ERROR: La fecha de nacimiento de la persona no puede ser nulo';
                'ERROR: El sexo no puede ser nulo'
                'ERROR: El valor del sexo es incorrecto. Valores correctos son 1-hombre, 2-mujer'
                'ERROR: La persona  no existe';
                'ERROR: (SQLERRM)'
   ******************************************************************************/
  PROCEDURE UPD_PERSONAS (pidperson IN PERSONAS.IDPERSON%TYPE,
                          pfnacimi  IN PERSONAS.FNACIMI%TYPE,
                          pcsexper  IN PERSONAS.CSEXPER%TYPE,
                          RSP       OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN

     PAR_IN := 'pidperson:'||pidperson||', pfnacimi: '||pfnacimi||', pcsexper: '||pcsexper;

     RSP := 'ERROR: El Identificador de la persona no puede ser nulo';
     IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de nacimiento de la persona no puede ser nulo';
     IF pfnacimi IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El sexo no puede ser nulo';
     IF pcsexper IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El valor del sexo es incorrecto. Valores correctos son 1-hombre, 2-mujer' ;
     IF pcsexper not in (1,2)  THEN  RAISE E_ERROR; END IF;

     --Verifica que la votacin exista en la Base de datos
     BEGIN
       SELECT 'S'
         INTO existe
         FROM PERSONAS
        WHERE IDPERSON = pidperson;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La persona  no existe';
           RAISE E_ERROR;
     END;

     --Si se pasan las pruebas de validacin actualiza
     UPDATE PERSONAS
        SET fnacimi  = pfnacimi,
            csexper  = pcsexper
      WHERE IDPERSON = pidperson;
     --
     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.UPD_PERSONAS', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.UPD_PERSONAS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.UPD_PERSONAS', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_PERSONAS;

  /******************************************************************************
   NOMBRE:      INS_PERSOCENSO
   DESCRIPCION: Inserta el censo de una persona. Si se cumplen que:
   							- Los campos obligatorios estn informados
   							- Exista la persona en la B.D. 
   							- Exista el pas y est vigente
   							- Exista la regin  y est vigente
   							- Exista la localidad  y est vigente
   							- Exista Pas/regin/localidad y est vigente
   							- No tenga un censo activo asignado, si es as primero tendr que
   							dar de baja ese censo activo.

   PARAMETROS:
    INPUT:  Obligatorios: - pidperson NUMBER(8): Identificacin de la persona
                          - pcodpais  VARCHAR(2): Cdigo del pas.
                          - pidregion NUMBER(8): identificador de la regin.
                          - pidlocali NUMBER(8): identificador de la localidad.
    OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No exista el registro
   POSTCONDICION:  Se da de alta el registro
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la persona no puede ser nulo'
                'ERROR: El Cdigo del pas no puede ser nulo'
                'ERROR: El Identificador de la regin no puede ser nulo'
                'ERROR: El Identificador de la localidad no puede ser nulo'
                'ERROR: La persona no existe'
                'ERROR: La localidad no existe para la Regin/Pais. o no est vigente'
                'ERROR: La Regin no existe para el Pas  o no est vigente'
                'ERROR: El Pas no existe  o no est vigente'
                'ERROR: Esta persona ya tiene asignado un censo';
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE INS_PERSOCENSO (pidperson IN PERSOCENSO.IDPERSON%TYPE,
                            pcodpais  IN PERSOCENSO.CODPAIS%TYPE,
                            pidregion IN PERSOCENSO.IDREGION%TYPE,
                            pidlocali IN PERSOCENSO.IDLOCALI%TYPE,
                            RSP      OUT VARCHAR2) IS
   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;

   BEGIN

     PAR_IN := 'pidperson: '||pidperson||', pcodpais:'||pcodpais||', pidregion: '||pidregion||', pidlocali: '||pidlocali;
     -- Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Identificador de la persona no puede ser nulo';
     IF pidperson IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El Cdigo del pas no puede ser nulo';
     IF pcodpais is NULL THEN RAISE E_ERROR; END IF;
     RSP := 'ERROR: El Identificador de la regin no puede ser nulo';
     IF pidregion IS NULL THEN RAISE E_ERROR; END IF;
     RSP := 'ERROR: El Identificador de la localidad no puede ser nulo';
     IF pidlocali IS NULL THEN  RAISE E_ERROR; END IF;

     -- Verifico que existan la persona
     BEGIN
 	    SELECT 'S'
        INTO existe
        FROM PERSONAS
       WHERE idperson = pidperson;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La persona no existe';
            RAISE E_ERROR;
     END;
     --
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM LOCALIDAD
        WHERE codpais = pcodpais
          AND idregion = pidregion
          AND idlocali = pidlocali
          AND fbaja is null;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La localidad no existe para la Regin/Pas. o no est vigente';
            RAISE E_ERROR;
     END;
     --
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM REGION
        WHERE codpais = pcodpais
          AND idregion = pidregion
          AND fbaja is null;
     EXCEPTION
         WHEN NO_DATA_FOUND THEN
              RSP := 'ERROR: La Regin no existe para el Pas o no est vigente';
              RAISE E_ERROR;
     END;
     --
     BEGIN
 	    SELECT 'S'
        INTO EXISTE
        FROM PAISES
       WHERE codpais = pcodpais
         AND fbaja is null;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: El Pas no existe o no est vigente';
            RAISE E_ERROR;
     END;
     --
     BEGIN
  	   SELECT 'S'
     	   INTO EXISTE
     	   FROM PERSOCENSO
     	  WHERE idperson = pidperson
     	    AND fbaja is null;
       RSP := 'ERROR: Esta persona ya tiene asignado un censo';
       RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;

     -- Si cumple condiones se graba el registro
     INSERT INTO PERSOCENSO (IDPERSON, CODPAIS, IDREGION, IDLOCALI, FALTA)
         VALUES(pidperson, pcodpais, pidregion, pidlocali, TRUNC(SYSDATE));

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.INS_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.INS_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.INS_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;
  END INS_PERSOCENSO;
  /******************************************************************************
   NOMBRE:      DEL_PERSOCENSO
   PROPSITO:   Da de baja un censo de una persona, si previamente supera los siguientes controles:
                 -El campo idpersona este informado
                 -La persona exista
                 -La persona tenga un censo asociado activo
    Si se pasa las pruebas de valicacin realiza el borrado Fsico del censo, si tiene
    dependencias se realizar un borrado lgico, informando la fecha de baja.

   PARAMETROS:
        INPUT:  Obligatorios: pidperson NUMBER(08): Identificador de la persona

        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista un censo activo para esa persona
   POSTCONDICION: Se da de baja (fsica/lgica) el censo activo para esa persona
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El identificador de persona no puede ser nulo'
                'ERROR: La persona no existe'
                'ERROR: No existe censo asociado activo para esta persona'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE DEL_PERSOCENSO(pidperson    IN PERSOCENSO.IDPERSON%TYPE,
                           RSP          OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);

   BEGIN

     PAR_IN := 'pidperson: '||pidperson;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El identificacin de persona no puede ser nulo';
     IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;

     -- Verifico que existan la persona
     BEGIN
 	    SELECT 'S'
        INTO existe
        FROM PERSONAS
       WHERE idperson = pidperson;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La persona no existe';
            RAISE E_ERROR;
     END;
     --Verifica que exista en la Base de datos
     BEGIN
       SELECT 1
         INTO existe
         FROM PERSOCENSO
        WHERE idperson = pidperson
          AND fbaja IS NULL;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: No existe censo asociado activo para esta persona';
           RAISE E_ERROR;
     END;

     --Si se pasa las pruebas de valicacin realiza el borrado Fsico/lgico del censo
     BEGIN
     	SELECT 'S' 
     	  INTO EXISTE
  			FROM VOTAPERSONA VP, VOTACENSO VC, PERSOCENSO PC
 			WHERE VP.idperson=pidperson
   			AND VP.idperson = PC.IDPERSON
   			AND PC.fbaja IS NULL
   			AND VP.idvota = VC.idvota
   			AND PC.codpais = NVL(VC.codpais,PC.codpais)
   			AND PC.idregion = NVL(VC.idregion,PC.idregion)
   			AND PC.idlocali = NVL(VC.idlocali,PC.idlocali)
   			AND ROWNUM  = 1;   
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            EXISTE := NULL;
     END;     	 
     IF EXISTE IS NULL THEN
        BEGIN
          DELETE PERSOCENSO WHERE  idperson = pidperson and fbaja IS NULL;
        EXCEPTION
          WHEN OTHERS THEN
               RSP := SUBSTR(SQLERRM,1,200);
               RAISE E_ERROR;
        END;
     ELSE
        BEGIN
        	 UPDATE PERSOCENSO SET fbaja = trunc(sysdate)
         	  WHERE idperson = pidperson
        	    AND fbaja IS NULL;
        EXCEPTION
           WHEN OTHERS THEN
                RSP := SUBSTR(SQLERRM,1,200);
                RAISE E_ERROR;
        END;
     END IF;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.DEL_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.DEL_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.DEL_PERSOCENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_PERSOCENSO;
  /******************************************************************************
   NOMBRE:      INS_VOTAPERSONA
   DESCRIPCION: Inserta la opcin escogida por una persona en una votacion, si cumple
                con los siguientes requisitos:
                - Que la persona corresponda al censo de la votacin.
		- La persona tenga 18 aos
                - Que la persona no haya votado en la votacin.
		- Que exista la persona
                - Que exista la votacin y la opcin escogida

   PARAMETROS:
    INPUT:  Obligatorios: - pidvota NUMBER(08): Identificador de votacin
                          - pidperson NUMBER(08): Identificacin de la persona
                          - nopcion  NUMBER(08): Numero de la opcin de la votacin.
    OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No exista el registro
   POSTCONDICION:  Se da de alta el registro
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la votacin no puede ser nulo'
                'ERROR: El Identificador de la persona no puede ser nulo'
                'ERROR: El numero de opcin  no puede ser nulo'
                'ERROR: La persona no puede votar no tiene de 18 aos'
                'ERROR: La persona no existe'
                'ERROR: La votacin no existe'
                'ERROR: La votacin no est en estado Abierto'
                'ERROR: La opcin para esta votacin no existe'
                'ERROR: Esta persona no puede votar por no pertenecer al censo electoral'
                'ERROR: Esta persona ya ha ejercido el derecho a voto en esta votacin'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE INS_VOTAPERSONA (pidvota   IN VOTAPERSONA.IDVOTA%TYPE,
                             pidperson IN VOTAPERSONA.IDPERSON%TYPE,
                             pnopcio   IN VOTAPERSONA.NOPCIO%TYPE,
                             RSP      OUT VARCHAR2) IS
   E_ERROR       EXCEPTION;
   --
   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lfnacimi      PERSONAS.FNACIMI%TYPE;
   lnedadvota    VOTAPERSONA.NEDADVOTA%TYPE;
   lcestado      VOTACION.cestado%TYPE;
   --
   BEGIN

     PAR_IN := 'pidvota: '||pidvota||', pidperson:'||pidperson||', pnopcio: '||pnopcio;
     -- Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Identificador de la votacin no puede ser nulo';
     IF pidvota IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El Identificador de la persona no puede ser nulo';
     IF pidperson IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El numero de opcin  no puede ser nulo';
     IF pnopcio IS NULL THEN RAISE E_ERROR; END IF;

     -- Verifico que existan la persona
     BEGIN
 	SELECT fnacimi
        INTO lfnacimi
        FROM PERSONAS
       WHERE idperson = pidperson;
       lnedadvota := ROUND(MONTHS_BETWEEN(TRUNC(SYSDATE), lfnacimi)/12,0);
       IF lnedadvota < 18 THEN
	   RSP := 'ERROR: La persona no puede votar no tiene de 18 aos';
           RAISE E_ERROR;
       END IF;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La persona no existe';
            RAISE E_ERROR;
     END;
     -- Verifico que existan la votacin
     BEGIN
 	    SELECT cestado
        INTO lcestado
        FROM VOTACION
       WHERE idvota = pidvota;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La votacin no existe';
            RAISE E_ERROR;
     END;
     --
     IF lcestado in (0,1,3) THEN
        RSP := 'ERROR: La votacin no est en estado Abierta';
        RAISE E_ERROR;
     END IF;
     -- Verifico que la opcin exista para la votacin
     BEGIN
 	SELECT 'S'
        INTO existe
        FROM OPCIONES
       WHERE idvota = pidvota
         and nopcio = pnopcio;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La opcin para esta votacin no existe';
            RAISE E_ERROR;
     END;
     -- Verifico que esta persona corresponda al censo de la votacin
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM PERSOCENSO PC, VOTACENSO VC
        WHERE PC.IDPERSON = pidperson
          AND VC.IDVOTA   = pidvota
          AND TRUNC(SYSDATE) BETWEEN PC.FALTA AND NVL(PC.FBAJA,TO_DATE('31122999','DDMMYYYY'))
          AND PC.CODPAIS  = NVL(VC.CODPAIS, PC.CODPAIS)
          AND PC.IDREGION = NVL(VC.IDREGION, PC.IDREGION)
          AND PC.IDLOCALI = NVL(VC.IDLOCALI, PC.IDLOCALI);
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: Esta persona no puede votar por no pertenecer al censo electoral';
            RAISE E_ERROR;
     END;
     -- Verifico que esta persona no ha votado en esta votacin
     BEGIN
       SELECT 'S'
         INTO EXISTE
         FROM VOTAPERSONA VP
        WHERE VP.IDPERSON = pidperson
          AND VP.IDVOTA   = pidvota;
       RSP := 'ERROR: Esta persona ya ha ejercido el derecho a voto en esta votacin';
       RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := NULL;
     END;
     -- Si cumple condiones se graba el registro
     INSERT INTO VOTAPERSONA (IDVOTA, IDPERSON, NOPCIO, FVOTA, NEDADVOTA)
         VALUES(pidvota, pidperson, pnopcio, TRUNC(SYSDATE), lnedadvota);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_PERSONAS.INS_VOTAPERSONA', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.INS_VOTAPERSONA', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_PERSONAS.INS_VOTAPERSONA', PAR_IN, 'RSP: '||RSP);

        COMMIT;
  END INS_VOTAPERSONA;

END PK_PERSONAS;
/
spool off;