spool 7-pk_votacion.log
prompt creando packages pk_votacion...
CREATE OR REPLACE PACKAGE PK_VOTACION AS
/******************************************************************************
   NOMBRE:   PK_VOTACION
   DESCRIPCIN: GESTION DE TODA LA INFORMACIN RELACIONADA CON LAS VOTACIONES.

******************************************************************************/

  /**************************************
   *  MANTENIMIENTO VOTACIONES
   **************************************/
  PROCEDURE INS_VOTACION (pttitulo   IN VOTACION.TTITULO%TYPE,
                          ptdescrip  IN VOTACION.TDESCRIP%TYPE,
                          ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                          pffecini   IN VOTACION.FFECINI%TYPE,
                          pffecfin   IN VOTACION.FFECFIN%TYPE,
                          pffecpub   IN VOTACION.FFECPUB%TYPE,
                          ptweb      IN VOTACION.TWEB%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE DEL_VOTACION (pidvota    IN VOTACION.IDVOTA%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE UPD_VOTACION (pidvota    IN VOTACION.IDVOTA%TYPE,
                          pttitulo   IN VOTACION.TTITULO%TYPE,
                          ptdescrip  IN VOTACION.TDESCRIP%TYPE,
                          ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                          pffecini   IN VOTACION.FFECINI%TYPE,
                          pffecfin   IN VOTACION.FFECFIN%TYPE,
                          pffecpub   IN VOTACION.FFECPUB%TYPE,
                          ptweb      IN VOTACION.TWEB%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE UPD_VOTACION (pidvota    IN VOTACION.IDVOTA%TYPE,
                          ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                          pffecfin   IN VOTACION.FFECFIN%TYPE,
                          ptweb      IN VOTACION.TWEB%TYPE,
                          RSP       OUT VARCHAR2);

  PROCEDURE UPD_ESTADO_VOTA (pidvota  IN VOTACION.IDVOTA%TYPE,
                             pcestado IN VOTACION.CESTADO%TYPE,
                             RSP OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO OPCIONES
   **************************************/

  PROCEDURE INS_OPCION (pidvota   IN OPCIONES.IDVOTA%TYPE,
                        pnopcio   IN OPCIONES.NOPCIO%TYPE,
                        ptopcion  IN OPCIONES.TOPCION%TYPE,
                        RSP       OUT VARCHAR2);

  PROCEDURE DEL_OPCION (pidvota   IN OPCIONES.IDVOTA%TYPE,
                        pnopcio   IN OPCIONES.NOPCIO%TYPE,
                        RSP       OUT VARCHAR2);

  PROCEDURE UPD_OPCION (pidvota   IN OPCIONES.IDVOTA%TYPE,
                        pnopcio   IN OPCIONES.NOPCIO%TYPE,
                        ptopcion  IN OPCIONES.TOPCION%TYPE,
                        RSP       OUT VARCHAR2);

  /**************************************
   *  MANTENIMIENTO CENSO - VOTACION
   **************************************/
  PROCEDURE INS_VOTACENSO (pidvota   IN VOTACENSO.IDVOTA%TYPE,
                           pcodpais  IN VOTACENSO.CODPAIS%TYPE,
                           pidregion IN VOTACENSO.IDREGION%TYPE,
                           pidlocali IN VOTACENSO.IDLOCALI%TYPE,
                           RSP      OUT VARCHAR2);

  PROCEDURE DEL_VOTACENSO (pidvota       IN VOTACENSO.IDVOTA%TYPE,
                           pidvotacenso  IN VOTACENSO.IDVOTACENSO%TYPE,
                           RSP      OUT VARCHAR2);
END PK_VOTACION;
/
CREATE OR REPLACE PACKAGE BODY PK_VOTACION AS

  /******************************************************************************
   NOMBRE:      INS_VOTACION
   DESCRIPCIN: Da de alta una votacion, si:
                  -Todos los campos obligatorios estan informados
                  -y las validaciones sean correctas
   PARAMETROS:
        INPUT:  Obligatorios: pttitulo VARCHAR2(20):  Titulo votacin
                              ptdescrip VARCHAR2(50): Descripcin votacin
                              ptnompresi VARCHAR2(50): Nombre del presidente
                              pffecini DATE: Fecha inicio votacin
                              pffecfin DATE: Fecha final votacin
                              pffecpub DATE: Fecha de publicacin de la votacin
                Opcionales:   ptweb VARCHAR2(30): Nombre de la web
       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No exista la votacin
   POSTCONDICION: Se da de alta la votacin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: Titulo votacin no puede ser nulo'
                'ERROR: La Descripcin de la votacin no puede ser nula'
                'ERROR: El nombre del presidente no puede ser nulo'
                'ERROR: La fecha de inicio de la votacin no puede ser nula'
                'ERROR: La fecha final de la votacin no puede ser nula'
                'ERROR: La fecha final no puede ser anterior a la fecha inicio'
                'ERROR: La fecha de publicacin no puede ser nula'
                'ERROR: La fecha de publicacin tiene que ser como mnimo una semana antes a la fecha de inicio'
                'ERROR: (SQLERRM)'

   ******************************************************************************/
   PROCEDURE INS_VOTACION (pttitulo   IN VOTACION.TTITULO%TYPE,
                           ptdescrip  IN VOTACION.TDESCRIP%TYPE,
                           ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                           pffecini   IN VOTACION.FFECINI%TYPE,
                           pffecfin   IN VOTACION.FFECFIN%TYPE,
                           pffecpub   IN VOTACION.FFECPUB%TYPE,
                           ptweb      IN VOTACION.TWEB%TYPE,
                           RSP       OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lttitulo      VOTACION.TTITULO%TYPE;


   BEGIN
     lttitulo :=TRIM(UPPER(pttitulo));
     PAR_IN := 'pttitulo: '||lttitulo||', ptdescrip: '||ptdescrip||', ptnompresi: '||ptnompresi||', pffecini: '||pffecini
             ||', pffecfin: '||pffecfin||', pffecpub: '||pffecpub||', ptweb: '||ptweb;

     --Verifica los parametros de entrada
     RSP := 'ERROR: Titulo votacin no puede ser nulo';
     IF lttitulo IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin de la votacin no puede ser nula';
     IF (TRIM(ptdescrip)       IS NULL) THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El nombre del presidente no puede ser nulo';
     IF (TRIM(ptnompresi)       IS NULL) THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de inicio de la votacin no puede ser nula';
     IF pffecini IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha final de la votacin no puede ser nula';
     IF pffecfin IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha final no puede ser anterior a la fecha inicio';
     IF pffecini > pffecfin THEN RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de publicacin no puede ser nula';
     IF pffecpub IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de publicacin tiene que ser como mnimo una semana antes a la fecha de inicio' ;
     IF (pffecpub+7) >  pffecini  THEN  RAISE E_ERROR; END IF;


     -- Pasa las validaciones e Inserta en la tabla
     INSERT INTO VOTACION (IDVOTA, TTITULO, TDESCRIP, TNOMPRESI, FFECINI, FFECFIN, FFECPUB, TWEB, CESTADO)
                  VALUES (seq_idvota.nextval, lttitulo, TRIM(ptdescrip),TRIM(ptnompresi), pffecini, pffecfin, pffecpub, ptweb, 0);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.INS_VOTACION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_VOTACION.INS_VOTACION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_VOTACION.INS_VOTACION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_VOTACION;

  /******************************************************************************
   NOMBRE:      DEL_VOTACION
   PROPSITO:   Da de baja una votacin, si previamente supera los siguientes controles:
                 -El campo idvota este informado
                 -La votacin exista en la Base de datos
                 -La votacin no est abierta, ni finalizada

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(8): Identificador de votacin
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la votacin
   POSTCONDICION: Se da de baja la votacin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El identificacin de votacin no puede ser nulo'
                'ERROR: La votacin no existe'
                'ERROR: No se puede dar de baja una votacin finalizada'
                'ERROR: No se puede dar de baja una votacin abierta'
                'ERROR: (SQLERRM)'
   ******************************************************************************/
  PROCEDURE DEL_VOTACION (pidvota  IN VOTACION.IDVOTA%TYPE,
                          RSP      OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN          VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;

   BEGIN

     PAR_IN := 'pidvota: '||pidvota;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El identificacin de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la votacin exista en la Base de datos
     BEGIN
       SELECT cestado
         INTO lcestado
         FROM VOTACION
        WHERE IDVOTA = pidvota;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin no existe';
           RAISE E_ERROR;
      END;

      -- Si est abierta o finalizada no se puede dar de baja
      IF lcestado = 3 THEN
         RSP := 'ERROR: No se puede dar de baja una votacin finalizada';
         RAISE E_ERROR;
      ELSIF lcestado = 2 THEN
         RSP := 'ERROR: No se puede dar de baja una votacin abierta';
      END IF;

      --Si se pasa las pruebas de valicacin realiza el borrado Fsico de la votacin
      BEGIN
        DELETE VOTACION WHERE  IDVOTA = pidvota;
      EXCEPTION
       WHEN OTHERS THEN
            BEGIN
            	DELETE OPCIONES WHERE IDVOTA = pidvota;
            	DELETE VOTACENSO WHERE IDVOTA = pidvota;
            EXCEPTION
              WHEN OTHERS THEN
                   RSP := SUBSTR(SQLERRM,1,200);
                   RAISE E_ERROR;
            END;
      END;

      RSP := 'OK';

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.DEL_VOTACION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_VOTACION;

  /******************************************************************************
   NOMBRE:      UPD_VOTACION
   DESCRIPCION: Realiza modificaciones en una votacin.
                Si estado de la votacin esta en Edicin o Pendiente. Se podr
                modificar todos los campos.
                Si estado de la votacin esta finalizado o abierto
                  -- No se podr modificar ningn dato.

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08): Identificador de la votacin
                              ttitulo VARCHAR2(20): Titulo de la votacin
                              ptdescrip VARCHAR2(50): Descripcin votacin
                              ptnompresi VARCHAR2(50): Nombre del presidente
                              pffecini DATE: Fecha inicio votacin
                              pffecfin DATE: Fecha final votacin
                              pffecpub DATE: Fecha de publicacin de la votacin
                Opcionales:   ptweb VARCHAR2(30): Nombre de la web
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la votacin a modificar
   POSTCONDICION: Se realiza la modificacin de la votacin
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la votacin no puede ser nulo'
                'ERROR: La votacin no existe'
                'ERROR: No se puede modificar una votacin finalizada'
                'ERROR: No se puede modificar todos los datos de una votacin abierta'
                'ERROR: Titulo votacin no puede ser nulo'
                'ERROR: La Descripcin de la votacin no puede ser nula'
                'ERROR: El nombre del presidente no puede ser nulo'
                'ERROR: La fecha de inicio de la votacin no puede ser nula'
                'ERROR: La fecha final de la votacin no puede ser nula'
                'ERROR: La fecha final no puede ser anterior a la fecha inicio'
                'ERROR: La fecha de publicacin no puede ser nula'
                'ERROR: La fecha de publicacin tiene que ser como mnimo una semana antes a la fecha de inicio'
                'ERROR: (SQLERRM)'
   ******************************************************************************/
  PROCEDURE UPD_VOTACION (pidvota  IN VOTACION.IDVOTA%TYPE,
                          pttitulo   IN VOTACION.TTITULO%TYPE,
                          ptdescrip  IN VOTACION.TDESCRIP%TYPE,
                          ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                          pffecini   IN VOTACION.FFECINI%TYPE,
                          pffecfin   IN VOTACION.FFECFIN%TYPE,
                          pffecpub   IN VOTACION.FFECPUB%TYPE,
                          ptweb      IN VOTACION.TWEB%TYPE,
                          RSP      OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;
   lttitulo      VOTACION.ttitulo%TYPE;

   BEGIN

     lttitulo :=TRIM(UPPER(pttitulo));
     PAR_IN := 'idvota:'||pidvota||', pttitulo: '||lttitulo||', ptdescrip: '||ptdescrip||
               ', ptnompresi: '||ptnompresi||', pffecini: '||pffecini||
               ', pffecfin: '||pffecfin||', pffecpub: '||pffecpub||', ptweb: '||ptweb;

     RSP := 'ERROR: El Identificador de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la votacin exista en la Base de datos
     BEGIN
       SELECT cestado
         INTO lcestado
         FROM VOTACION
        WHERE IDVOTA = pidvota;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin  no existe';
           RAISE E_ERROR;
     END;

     IF lcestado = 3 THEN -- Finalizado
        RSP := 'ERROR: No se puede modificar una votacin finalizada';
        RAISE E_ERROR;
     END IF;

     IF lcestado = 2 THEN -- Abierta
        RSP := 'ERROR: No se puede modificar todos los datos de una votacin abierta';
        RAISE E_ERROR;
     END IF;

     --Verifica los parametros de entrada
     RSP := 'ERROR: Titulo votacin no puede ser nulo';
     IF lttitulo IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La Descripcin de la votacin no puede ser nula';
     IF (TRIM(ptdescrip)       IS NULL) THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El nombre del presidente no puede ser nulo';
     IF (TRIM(ptnompresi)       IS NULL) THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de inicio de la votacin no puede ser nula';
     IF pffecini IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha final de la votacin no puede ser nula';
     IF pffecfin IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha final no puede ser anterior a la fecha inicio';
     IF pffecini > pffecfin THEN RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de publicacin no puede ser nula';
     IF pffecpub IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha de publicacin tiene que ser como mnimo una semana antes a la fecha de inicio' ;
     IF (pffecpub+7) >  pffecini  THEN  RAISE E_ERROR; END IF;

     --Si se pasan las pruebas de validacin actualiza
     UPDATE VOTACION
        SET TTITULO  = lttitulo,
            TDESCRIP = TRIM(ptdescrip),
            TNOMPRESI = TRIM(ptnompresi),
            FFECINI   = pffecini,
            FFECFIN   = pffecfin,
            FFECPUB   = pffecpub,
            TWEB      = ptweb
      WHERE IDVOTA = pidvota;
     --
     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_VOTACION;

  /******************************************************************************
   NOMBRE:      UPD_VOTACION
   DESCRIPCION: Realiza modificaciones en la votacin.
                Se podr modificar:
                  -- Fecha finalizacin de la votacin
                  -- Nombre del presidente de mesa
                  -- Direccin Web
                Si estado de la votacin esta finalizado
                  -- No se podr modificar ningn dato.

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08): Identificador de la votacin
                              ptnompresi VARCHAR2(50): Nombre del presidente
                              pffecfin DATE: Fecha final votacin
                Opcionales:   ptweb VARCHAR2(30): Nombre de la web
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Existe la votacin a modificar
   POSTCONDICION: Se realiza la modificacin de la votacin
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la votacin no puede ser nulo'
                'ERROR: La votacin no existe'
                'ERROR: No se puede modificar una votacin finalizada'
                'ERROR: El nombre del presidente no puede ser nulo'
                'ERROR: La fecha final de la votacin no puede ser nula'
                'ERROR: La fecha final no puede ser anterior a la fecha inicio'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE UPD_VOTACION (pidvota  IN VOTACION.IDVOTA%TYPE,
                          ptnompresi IN VOTACION.TNOMPRESI%TYPE,
                          pffecfin IN VOTACION.FFECFIN%TYPE,
                          ptweb      IN VOTACION.TWEB%TYPE,
                          RSP      OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;

   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;
   lffecini      VOTACION.ffecini%TYPE;

   BEGIN

     PAR_IN := 'pidvota:'||pidvota||', pffecfin:'||pffecfin||', ptnompresi:'||ptnompresi||
               ', ptweb:'||ptweb;

     RSP := 'ERROR: El Identificador de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la votacin exista en la Base de datos
     BEGIN
       SELECT cestado, ffecini
         INTO lcestado, lffecini
         FROM VOTACION
        WHERE IDVOTA = pidvota;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin  no existe';
           RAISE E_ERROR;
     END;

     IF lcestado = 3 THEN -- Finalizado
        RSP := 'ERROR: No se puede modificar una votacin finalizada';
        RAISE E_ERROR;
     END IF;

     RSP := 'ERROR: El nombre del presidente no puede ser nulo';
     IF (TRIM(ptnompresi) IS NULL) THEN  RAISE E_ERROR; END IF;

     RSP := 'ERROR: La fecha final de la votacin no puede ser nula';
     IF pffecfin IS NULL  THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: La fecha final no puede ser anterior a la fecha inicio';
     IF lffecini > pffecfin THEN RAISE E_ERROR; END IF;


     --Si se pasan las pruebas de validacin actualiza
     UPDATE VOTACION
        SET TNOMPRESI = TRIM(ptnompresi),
            FFECFIN   = pffecfin,
            TWEB      = TRIM(ptweb)
      WHERE IDVOTA = pidvota;



     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_VOTACION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_VOTACION;
  /******************************************************************************
   NOMBRE:      UPD_ESTADO_VOTA
   DESCRIPCION: Realiza modificaciones en el estado de una votacin. Si:
                -- Existe votacin
                -- No se podr modificar si el estado esta finalizado.
		-- El estado modificado no puede ser menor que el actual.

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08): Identificador de la votacin
                              pcestado NUMBER(02): Estado de la votacin
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Existe la votacin a modificar
   POSTCONDICION: Se realiza la modificacin del estado la votacin
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Identificador de la votacin no puede ser nulo'
                'ERROR: La votacin no existe'
		'ERROR: No se puede cambiar el estado, como mnimo tiene que existir dos opciones'
                'ERROR: No se puede modificar una votacin finalizada'
                'ERROR: No se puede retroceder el estado de la votacin a uno inferior';
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE UPD_ESTADO_VOTA (pidvota  IN VOTACION.IDVOTA%TYPE,
                             pcestado IN VOTACION.CESTADO%TYPE,
                             RSP OUT VARCHAR2) IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lffecini      VOTACION.FFECINI%TYPE;
   lpotvota      PCIUNOVOT_ANYO.NTOTPER%TYPE;
   lnunhanvot    PCIUNOVOT_ANYO.NTOTNVOT%TYPE;
   lpnovota      PCIUNOVOT_ANYO.PPERNVOT%TYPE;
   ltotopcio     NUMBER;
   lcestado      VOTACION.cestado%type;

   BEGIN
     PAR_IN := 'pidvota:'||pidvota||', pcestado:'||pcestado;

     RSP := 'ERROR: El Identificador de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la votacin exista en la Base de datos
     BEGIN
       SELECT cestado, ffecini
         INTO lcestado, lffecini
         FROM VOTACION
        WHERE IDVOTA = pidvota;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin  no existe';
           RAISE E_ERROR;
     END;

    --Verifica que la votacin tenga ms de dos opciones para escoger
     BEGIN
       SELECT count(nopcio)
         INTO ltotopcio
         FROM OPCIONES
        WHERE IDVOTA = pidvota;
     EXCEPTION
        WHEN others THEN
           ltotopcio := 0;
           RSP := 'ERROR: Error al consultar opciones:'||sqlerrm;
           RAISE E_ERROR;
     END;

     IF nvl(ltotopcio,0) < 2 THEN
        RSP := 'ERROR: No se puede cambiar el estado, como mnimo tiene que existir dos opciones';
        RAISE E_ERROR;
     END IF;

     IF lcestado = 3 THEN -- Finalizado
        RSP := 'ERROR: No se puede modificar una votacin finalizada';
        RAISE E_ERROR;
     END IF;

     IF pcestado < lcestado THEN
        RSP := 'ERROR: No se puede retroceder el estado de la votacin a uno inferior';
        RAISE E_ERROR;
     END IF;
     --
     IF pcestado = 3 THEN
        BEGIN
          -- Calculo el numero de personas con capacidad de voto y personas que han votado
          UPDATE VOTACION v set ncapvot = (SELECT COUNT(1) FROM PERSONAS p, PERSOCENSO pc, votacenso vc
                                            WHERE vc.idvota = v.idvota
                                              AND (MONTHS_BETWEEN(v.ffecfin, p.FNACIMI)/12) >= 18
                                              AND p.idperson = pc.idperson
                                              AND v.ffecini between pc.falta and nvl(pc.fbaja,to_date('31122999','DDMMYYYY'))
                                              AND pc.codpais between nvl(vc.codpais,'AA') AND nvl(vc.codpais,'ZZ')
                                              AND pc.idregion between nvl(vc.idregion,'0') and nvl(vc.idregion,'99999999')
                                              AND pc.idlocali between nvl(vc.idlocali,'0') and nvl(vc.idlocali,'99999999')),
                             nvottot  = (SELECT count(1) FROM VOTAPERSONA WHERE idvota = v.idvota)
           WHERE v.idvota = pidvota;

          -- calculo del % de participacion
          UPDATE VOTACION v SET pparti = (nvottot / ncapvot)*100 WHERE v.idvota = pidvota;

          -- Recuento de votos.
          -- Actualiza nro de votos obtenidos por opcin
          UPDATE OPCIONES op SET nnumvot = (SELECT count(1)
                                             FROM VOTAPERSONA
                                            WHERE idvota = op.idvota
                                              AND nopcio = op.nopcio)
           WHERE op.idvota = pidvota;
          -- Actualiza % de votos por opcin respecto al total
          UPDATE OPCIONES op SET prestot = (nnumvot /
                                        (SELECT nvottot FROM VOTACION where idvota = op.idvota))*100
           WHERE op.idvota = pidvota;
        EXCEPTION
          WHEN OTHERS THEN
               ROLLBACK;
               err_num := SQLCODE;
               err_msg := SUBSTR(SQLERRM, 1, 200);

               RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

               INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
                  VALUES(SYSDATE, 'PK_VOTACION.UPD_ESTADO_VOTA', PAR_IN, 'RSP: '||RSP);

               COMMIT;
        END;
        --
        BEGIN
          -- ------------------------------------------------------------------------
          -- Consulta 6
          -- % de ciudadanos que nunca han votado por ao
          -- ------------------------------------------------------------------------
          -- Nunca han votado
          SELECT COUNT(COUNT(pc.idperson))
            INTO lnunhanvot
            FROM PERSOCENSO pc, VOTACENSO vc, VOTACION v
           WHERE pc.codpais BETWEEN NVL(vc.codpais, 'AA') AND NVL(vc.codpais,'ZZ')
             AND pc.idregion BETWEEN NVL(vc.idregion, 0) AND NVL(vc.idregion,99999999)
             AND pc.idlocali BETWEEN NVL(vc.idlocali, 0) AND NVL(vc.idlocali,99999999)
             AND (v.ffecini > pc.falta and (pc.fbaja is null or pc.fbaja > v.FFECINI))
             AND vc.idvota = v.idvota
             AND to_number(TO_CHAR(v.FFECINI,'YYYY')) = to_number(TO_CHAR(lffecini,'YYYY'))
             AND pc.idperson NOT IN (SELECT idperson FROM VOTAPERSONA WHERE idperson = pc.idperson)
          GROUP BY pc.idperson;

          -- Pueden votar
          SELECT COUNT(COUNT(pc.idperson))
            INTO lpotvota
            FROM PERSOCENSO pc, VOTACENSO vc, VOTACION v
           WHERE pc.codpais BETWEEN NVL(vc.codpais, 'AA') AND NVL(vc.codpais,'ZZ')
             AND pc.idregion BETWEEN NVL(vc.idregion, 0) AND NVL(vc.idregion,99999999)
             AND pc.idlocali BETWEEN NVL(vc.idlocali, 0) AND NVL(vc.idlocali,99999999)
             AND vc.idvota = v.idvota
             AND to_number(TO_CHAR(v.FFECINI,'YYYY')) = to_number(TO_CHAR(lffecini,'YYYY'))
          GROUP BY pc.idperson;
          --
          lpnovota :=  round((lnunhanvot/lpotvota) * 100, 2);
          --
          BEGIN
          	INSERT INTO PCIUNOVOT_ANYO (NANYO, NTOTPER, NTOTNVOT, PPERNVOT)
          	  VALUES (to_number(TO_CHAR(lffecini,'YYYY')), lpotvota, lnunhanvot, lpnovota);
          EXCEPTION
            WHEN DUP_VAL_ON_INDEX THEN
                 UPDATE PCIUNOVOT_ANYO
                    SET NTOTPER = lpotvota,
                        NTOTNVOT = lnunhanvot,
                        PPERNVOT = lpnovota
                  WHERE nanyo = to_number(TO_CHAR(lffecini,'YYYY'));
          END;
          --
       EXCEPTION
         WHEN OTHERS THEN
              NULL;
       END;
     END IF;
     --
     UPDATE VOTACION SET CESTADO = pcestado
      WHERE idvota = pidvota;
     --
     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.UPD_ESTADO_VOTA', PAR_IN, 'RSP: '||RSP);

     COMMIT;
     --
   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_ESTADO_VOTA', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        ROLLBACK;
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_ESTADO_VOTA', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_ESTADO_VOTA;
  /******************************************************************************
   NOMBRE:      INS_OPCION
   DESCRIPCIN: Da de alta una opcin de una votacin, debe cumplir que:
                  -La votacin exista
                  -y las validaciones sean correctas
   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08):  identificador de votacin
                              pnopcio NUMBER(08):  Numero de opcin
                              ptopcion VARCHAR2(50): Descripcin de la opcin
        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No exista la opcin para la votacin
   POSTCONDICION: Se da de alta la opcin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: Identificador de votacin no puede ser nulo'
                'ERROR: El identificador de opcin no puede ser nulo'
                'ERROR: Descripcin de la opcin no puede ser nula'
                'ERROR: La votacin no existe'
                'ERROR: No se puede aadir una opcin a una votacin finalizada'
                'ERROR: No se puede aadir una opcin a una votacin abierta'
                'ERROR: (SQLERRM) '

   ******************************************************************************/
  PROCEDURE INS_OPCION (pidvota   IN OPCIONES.IDVOTA%TYPE,
                        pnopcio   IN OPCIONES.NOPCIO%TYPE,
                        ptopcion  IN OPCIONES.TOPCION%TYPE,
                        RSP       OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;
   lcestado      VOTACION.cestado%TYPE;


   BEGIN

     PAR_IN := 'pidvota: '||pidvota||', pnopcio: '||pnopcio||', ptopcion: '||ptopcion;

     --Verifica los parametros de entrada
     RSP :=  'ERROR: Identificador de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El identificador de opcin no puede ser nulo';
     IF pnopcio IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: Descripcin de la opcin no puede ser nula';
     IF (TRIM(ptopcion) IS NULL) THEN  RAISE E_ERROR; END IF;

     -- Verifica si existe la votacin.
     BEGIN
     	 SELECT cestado
     	   INTO lcestado
     	   FROM VOTACION
     	  WHERE idvota = pidvota;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La votacin no existe';
            RAISE E_ERROR;
     END;

     -- Si est abierta o finalizada no se puede dar de Alta
     IF lcestado = 3 THEN
        RSP := 'ERROR: No se puede aadir una opcin a una votacin finalizada';
        RAISE E_ERROR;
     ELSIF lcestado = 2 THEN
        RSP := 'ERROR: No se puede aadir una opcin a una votacin abierta';
     END IF;
     -- Verifica si existe la opcion.
     BEGIN
     	 SELECT 'S'
     	   INTO existe
     	   FROM OPCIONES
     	  WHERE idvota = pidvota
     	    AND nopcio = pnopcio;
       RSP := 'ERROR: La opcin ya ha sido dado de alta';
       RAISE E_ERROR;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            NULL;
     END;

     -- Insercin en la tabla
     INSERT INTO OPCIONES (idvota, nopcio, topcion) VALUES (pidvota, pnopcio, TRIM(ptopcion));

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.INS_OPCION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

  EXCEPTION
    WHEN E_ERROR THEN

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_VOTACION.INS_OPCION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

    WHEN OTHERS THEN
      err_num := SQLCODE;
      err_msg := SUBSTR(SQLERRM, 1, 200);

      RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
         VALUES(SYSDATE, 'PK_VOTACION.INS_OPCION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

  END INS_OPCION;
  /******************************************************************************
   NOMBRE:      DEL_OPCION
   PROPSITO:   Da de baja una opcion de una votacin, si previamente supera los siguientes controles:
                 -El campo idvota y nopcio este informado
                 -La votacin/opcion exista en la Base de datos
                 -La votacin no est abierta, ni finalizada

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08): Identificador de votacin
                              pnopcio NUMBER(08): Numero de opcin

        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la opcin para esa votacin
   POSTCONDICION: Se da de baja la opcin para la votacin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El identificacin de votacin no puede ser nulo'
                'ERROR: El identificacin de opcin no puede ser nulo'
                'ERROR: La opcin no existe para esta votacin'
                'ERROR: La votacin no existe'
                'ERROR: No se puede dar de baja una opcin de una votacin finalizada'
                'ERROR: No se puede dar de baja una opcin de una votacin abierta'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE DEL_OPCION (pidvota  IN OPCIONES.IDVOTA%TYPE,
                        pnopcio  IN OPCIONES.NOPCIO%TYPE,
                        RSP      OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;

   BEGIN

     PAR_IN := 'pidvota: '||pidvota;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El identificacin de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El identificacin de opcin no puede ser nulo';
     IF pnopcio IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la opcion exista en la Base de datos
     BEGIN
       SELECT 1
         INTO existe
         FROM OPCIONES
        WHERE IDVOTA = pidvota
          AND NOPCIO = pnopcio;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La opcin no existe para esta votacin';
           RAISE E_ERROR;
      END;

     -- Recoge el estado de la  votacin
     BEGIN
       SELECT cestado
         INTO lcestado
         FROM VOTACION
        WHERE IDVOTA = pidvota;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin no existe';
           RAISE E_ERROR;
      END;

      -- Si est abierta o finalizada no se puede dar de baja
      IF lcestado = 3 THEN
         RSP := 'ERROR: No se puede dar de baja una opcin de una votacin finalizada';
         RAISE E_ERROR;
      ELSIF lcestado = 2 THEN
         RSP := 'ERROR: No se puede dar de baja una opcin de una votacin abierta';
      END IF;

      --Si se pasa las pruebas de valicacin realiza el borrado Fisico de la opcion
      BEGIN
        DELETE OPCIONES WHERE  IDVOTA = pidvota and NOPCIO = pnopcio;
      EXCEPTION
       WHEN OTHERS THEN
            RSP := SUBSTR(SQLERRM,1,200);
            RAISE E_ERROR;
      END;

      RSP := 'OK';

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.DEL_OPCION', PAR_IN, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_OPCION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_OPCION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_OPCION;
  /******************************************************************************
   NOMBRE:      UPD_OPCION
   DESCRIPCIN: Modifica la descripcin de una opcin que pertenece a una votacin,
                debe cumplir que:
                  - Los campo obligatorios estn informados
                  - La opcin para a votacin exista
                  - La votacin no est abierta ni finalizada
   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08):  identificador de votacin
                              pnopcio NUMBER(08):  Numero de opcin
                              ptopcion VARCHAR2(50): Descripcin de la opcin
       OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista la opcin para la votacin
   POSTCONDICION: Se modifica la opcin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERROR' Si:
                'ERROR: Identificador de votacin no puede ser nulo'
                'ERROR: El identificador de opcin no puede ser nulo'
                'ERROR: Descripcin de la opcin no puede ser nula'
                'ERROR: La opcion no existe para esta votacion'
                'ERROR: La votacin no existe'
                'ERROR: No se puede modificar una opcin de una votacin finalizada'
                'ERROR: No se puede modificar una opcin a una votacin abierta'
                'ERROR: (SQLERRM) '
  ******************************************************************************/
  PROCEDURE UPD_OPCION (pidvota   IN OPCIONES.IDVOTA%TYPE,
                        pnopcio   IN OPCIONES.NOPCIO%TYPE,
                        ptopcion  IN OPCIONES.TOPCION%TYPE,
                        RSP       OUT VARCHAR2)  IS

   E_ERROR       EXCEPTION;
   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   num_err       NUMBER;

   lcestado      VOTACION.cestado%type;

   BEGIN
     PAR_IN := 'pidvota: '||pidvota||', pnopcio: '||pnopcio||', ptopcion: '||ptopcion;

     --Verifica los parametros de entrada
     RSP :=  'ERROR: Identificador de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El identificador de opcin no puede ser nulo';
     IF pnopcio IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: Descripcin de la opcin no puede ser nula';
     IF (TRIM(ptopcion) IS NULL) THEN  RAISE E_ERROR; END IF;

     --Verifica que la opcion exista en la Base de datos
     BEGIN
       SELECT 1
         INTO existe
         FROM OPCIONES
        WHERE IDVOTA = pidvota
          AND NOPCIO = pnopcio;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La opcin no existe para esta votacin';
           RAISE E_ERROR;
      END;

     -- Recoge el estado de la  votacin
     BEGIN
       SELECT cestado
         INTO lcestado
         FROM VOTACION
        WHERE IDVOTA = pidvota;
     EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin no existe';
           RAISE E_ERROR;
     END;

     IF lcestado = 3 THEN -- Finalizado
        RSP := 'ERROR: No se puede modificar una opcin de una votacin finalizada';
        RAISE E_ERROR;
     END IF;

     IF lcestado = 2 THEN -- Abierta
        RSP := 'ERROR: No se puede modificar una opcin de una votacin abierta';
        RAISE E_ERROR;
     END IF;

     UPDATE OPCIONES SET TOPCION = ptopcion
      WHERE idvota = pidvota
        AND nopcio = pnopcio;

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.UPD_OPCION', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_OPCION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.UPD_OPCION', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END UPD_OPCION;
  /******************************************************************************
   NOMBRE:      INS_VOTACENSO
   DESCRIPCION: Inserta el censo/s al cual va dirigida la votacin, si cumple que:
		- La votacin est informada y exista.
		- La votacin no est abierta ni finalizada
		- Si est informado el cdigo de Pas que exista y est vigente.
		- Si est informada la regin que exista y est vigente.
		- Si est informada la localidad que exista y est vigente.


   PARAMETROS:
    INPUT:  Obligatorios: - pidvota NUMBER(08): Identificacin votacin
            Opcionales:
                          - pcodpais VARCHAR2(02): Cdigo del pas.
                            NULL - Todo el censo.
                            Si es NULL no se puede informar la regin ni la localidad.
                          - pidregion NUMBER (08): identificador de la regin.
                            NULL - TODAS las regiones que pertenezcan al pais o censo global.
                            Si no es Null Tiene que informarse el pais
                            Si es NULL no puede informarse la localidad.
                          - pidlocali NUMBER (08): identificador de la localidad.
                            NULL - TODAS las localidades que pertenezcan a la regin o pais o censo global.

    OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: No exista el registro
   POSTCONDICION:  Se da de alta el registro
   RETORNA:
        Si ha ido bien:
                'OK' si la operacin se ha realizado con xito
        Si NO ha ido bien:
                'ERROR: El Cdigo de la votacin no puede ser nulo'
                'ERROR: El Identificador de la regin no puede ser nulo'
                'ERROR: El Cdigo del pas no puede ser nulo'
                'ERROR: La votacin no existe'
                'ERROR: No se puede aadir un censo a una votacin finalizada'
                'ERROR: No se puede aadir un censo a una votacin abierta'
                'ERROR: La localidad no existe para la Regin/Pas  o no est vigente'
                'ERROR: La Regin no existe para el Pas o no est vigente'
                'ERROR: El Pas no existe o no est vigente'
                'ERROR: El censo para esta votacin ya existe o esta contemplado';
                'ERROR: El censo para esta votacin ya esta contemplado'
                'ERROR: El censo es ms genrico que los existentes,
                        si desea uno genrico borre los anteriores'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE INS_VOTACENSO (pidvota   IN VOTACENSO.IDVOTA%TYPE,
                           pcodpais  IN VOTACENSO.CODPAIS%TYPE,
                           pidregion IN VOTACENSO.IDREGION%TYPE,
                           pidlocali IN VOTACENSO.IDLOCALI%TYPE,
                           RSP      OUT VARCHAR2) IS
   E_ERROR       EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(700);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;
   lffecini      VOTACION.ffecini%TYPE;
   lffecfin      VOTACION.ffecfin%TYPE;

   BEGIN

     PAR_IN := 'pidvota: '||pidvota||', pcodpais:'||pcodpais||', pidregion: '||pidregion||', pidlocali: '||pidlocali;
     -- Verifica que todos los campos obligatorios tengan valor
     RSP := 'ERROR: El Cdigo de la votacin no puede ser nulo';
     IF (pidvota IS NULL)  THEN  RAISE E_ERROR; END IF;

     -- Verifica que los campos no informados tengan sentido. Si se informa localidad tiene que informarse pas/regin
     -- Si se informa regin tiene que informarse pas.
     IF pidlocali IS NOT NULL THEN
        IF pidregion IS NOT NULL THEN
           IF pcodpais IS NULL THEN
               RSP := 'ERROR: El Cdigo del pas no puede ser nulo';
               RAISE E_ERROR;
           END IF;
        ELSE
           RSP := 'ERROR: El Identificador de la regin no puede ser nulo';
           RAISE E_ERROR;
        END IF;
     END IF;

     -- Verifico que existan
     BEGIN
 	    SELECT cestado, ffecini, ffecfin
        INTO lcestado, lffecini, lffecfin
        FROM VOTACION
       WHERE idvota = pidvota;
     EXCEPTION
       WHEN NO_DATA_FOUND THEN
            RSP := 'ERROR: La votacin no existe';
            RAISE E_ERROR;
     END;
     -- Si est abierta o finalizada no se puede dar de Alta nuevos censos
     IF lcestado = 3 THEN
        RSP := 'ERROR: No se puede aadir un censo a una votacin finalizada';
        RAISE E_ERROR;
     ELSIF lcestado = 2 THEN
        RSP := 'ERROR: No se puede aadir un censo a una votacin abierta';
     END IF;
     --
     IF pidlocali IS NOT NULL THEN
        BEGIN
     	    SELECT 'S'
            INTO EXISTE
            FROM LOCALIDAD
           WHERE codpais = pcodpais
             AND idregion = pidregion
             AND idlocali = pidlocali
             AND NVL(fbaja, to_Date('31122999','ddmmyyyy')) > lffecfin;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
               RSP := 'ERROR: La localidad no existe para la Regin/Pas o no est vigente';
               RAISE E_ERROR;
        END;
     END IF;
     --
     IF pidregion IS NOT NULL THEN
        BEGIN
     	    SELECT 'S'
            INTO EXISTE
            FROM REGION
           WHERE codpais = pcodpais
             AND idregion = pidregion
             AND NVL(fbaja, to_Date('31122999','ddmmyyyy')) > lffecfin;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
               RSP := 'ERROR: La Regin no existe para el Pas o no est vigente';
               RAISE E_ERROR;
        END;
     END IF;
     --
     IF pcodpais IS NOT NULL THEN
        BEGIN
     	    SELECT 'S'
            INTO EXISTE
            FROM PAISES
           WHERE codpais = pcodpais
             AND NVL(fbaja, to_Date('31122999','ddmmyyyy')) > lffecfin;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
               RSP := 'ERROR: El Pas no existe o no est vigente';
               RAISE E_ERROR;
        END;
     END IF;
     --
     -- Miro que los datos del censo sean coherentes. Si existe un registro
     -- ms generico, que el que se quiere introducir.
     -- Es decir, si existe uno que es toda Espaa, es ilogico que se grave
     -- uno de solo catalua.
     --
     IF pidlocali IS NOT NULL THEN
        BEGIN
     	    SELECT 'S'
     	      INTO EXISTE
     	      FROM VOTACENSO
     	     WHERE idvota = pidvota
     	       AND pcodpais = NVL(codpais,pcodpais)
     	       AND pidregion = NVL(idregion,pidregion)
     	       AND pidlocali = NVL(idlocali,pidlocali);
          RSP := 'ERROR: El censo para esta votacin ya existe o esta contemplado';
          RAISE E_ERROR;
        EXCEPTION
          WHEN NO_DATA_FOUND THEN
               NULL;
        END;
     ELSE
        IF pidregion IS NOT NULL THEN
           BEGIN
     	       SELECT 'S'
     	         INTO EXISTE
     	         FROM VOTACENSO
     	        WHERE idvota = pidvota
     	          AND pcodpais = NVL(codpais,pcodpais)
     	          AND pidregion = NVL(idregion,pidregion);
             RSP := 'ERROR: El censo para esta votacin ya esta contemplado';
             RAISE E_ERROR;
           EXCEPTION
             WHEN NO_DATA_FOUND THEN
                  NULL;
           END;
        ELSE
           IF pcodpais IS NOT NULL THEN
              BEGIN
     	          SELECT 'S'
     	            INTO EXISTE
     	            FROM VOTACENSO
     	           WHERE idvota = pidvota
     	             AND pcodpais = NVL(codpais,pcodpais);
     	          RSP := 'ERROR: El censo para esta votacin ya esta contemplado';
                RAISE E_ERROR;
              EXCEPTION
               WHEN NO_DATA_FOUND THEN
                    NULL;
              END;
           ELSE
              BEGIN
     	          SELECT 'S'
     	            INTO EXISTE
     	            FROM VOTACENSO
     	           WHERE idvota = pidvota
     	             AND codpais = nvl(pcodpais, codpais)
     	             AND ROWNUM   = 1;
                RSP := 'ERROR: El censo es ms genrico que los existentes, si desea uno genrico borre los anteriores';
                RAISE E_ERROR;
              EXCEPTION
                WHEN NO_DATA_FOUND THEN
                     NULL;
              END;
           END IF;
        END IF;
     END IF;

     -- Si cumple condiones se graba el registro
     INSERT INTO VOTACENSO (IDVOTACENSO, IDVOTA, CODPAIS, IDREGION, IDLOCALI)
         VALUES(seq_idvotacenso.nextval, pidvota, pcodpais, pidregion, pidlocali);

     RSP := 'OK';

     INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.INS_VOTACENSO', PAR_IN, 'RSP: '||RSP);

     COMMIT;

   EXCEPTION

     WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.INS_VOTACENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.INS_VOTACENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;
  END INS_VOTACENSO;
  /******************************************************************************
   NOMBRE:      DEL_VOTACENSO
   PROPSITO:   Da de baja un censo de una votacin, si previamente supera los siguientes controles:
                 -El campo idvotacenso/idvota este informado
                 -El censo exista en la Base de datos
                 -La votacin no est abierta, ni finalizada

   PARAMETROS:
        INPUT:  Obligatorios: pidvota NUMBER(08): Identificador de la votacin
                              pidvotacenso NUMBER(08): Identificador del censo de la votacin

        OUTPUT:  RSP VARCHAR2(300)
   PRECONDICION: Exista el censo para esa votacin
   POSTCONDICION: Se da de baja el censo para la votacin
   RETORNA:
        - 'OK' si la operacin se ha realizado con xito
        - 'ERRORES'
                'ERROR: El identificacin de votacin no puede ser nulo'
                'ERROR: El identificacin de votacin de censo no puede ser nulo'
                'ERROR: El censo no existe para esta votacin'
                'ERROR: La votacin no existe'
                'ERROR: No se puede dar de baja un censo de una votacin finalizada'
                'ERROR: No se puede dar de baja un censo de una votacin abierta'
                'ERROR: (SQLERRM)'
  ******************************************************************************/
  PROCEDURE DEL_VOTACENSO (pidvota      IN VOTACENSO.IDVOTA%TYPE,
                           pidvotacenso IN VOTACENSO.IDVOTACENSO%TYPE,
                           RSP          OUT VARCHAR2) IS

   E_ERROR EXCEPTION;

   existe        VARCHAR2(1);
   PAR_IN        VARCHAR2(500);
   err_num       NUMBER;
   err_msg       VARCHAR2(200);
   lcestado      VOTACION.cestado%TYPE;

   BEGIN

     PAR_IN := 'pidvota: '||pidvota||', pidvotacenso:'||pidvotacenso;

     --Verifica que el campo de busqueda este informado
     RSP := 'ERROR: El identificacin de votacin no puede ser nulo';
     IF pidvota IS NULL THEN  RAISE E_ERROR; END IF;
     RSP := 'ERROR: El identificacin de votacin de censo no puede ser nulo';
     IF pidvotacenso IS NULL THEN  RAISE E_ERROR; END IF;

     --Verifica que la opcion exista en la Base de datos
     BEGIN
       SELECT 1
         INTO existe
         FROM VOTACENSO
        WHERE IDVOTA = pidvota
          AND IDVOTACENSO = pidvotacenso;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: El censo no existe para esta votacin';
           RAISE E_ERROR;
      END;

     -- Recoge el estado de la  votacin
     BEGIN
       SELECT cestado
         INTO lcestado
         FROM VOTACION
        WHERE IDVOTA = pidvota;
      EXCEPTION
        WHEN NO_DATA_FOUND THEN
           RSP := 'ERROR: La votacin no existe';
           RAISE E_ERROR;
      END;

      -- Si est abierta o finalizada no se puede dar de baja
      IF lcestado = 3 THEN
         RSP := 'ERROR: No se puede dar de baja un censo de una votacin finalizada';
         RAISE E_ERROR;
      ELSIF lcestado = 2 THEN
         RSP := 'ERROR: No se puede dar de baja un censo de una votacin abierta';
      END IF;

      --Si se pasa las pruebas de valicacin realiza el borrado Fisico del censo
      BEGIN
        DELETE VOTACENSO WHERE  IDVOTA = pidvota and IDVOTACENSO = pidvotacenso;
      EXCEPTION
       WHEN OTHERS THEN
            RSP := SUBSTR(SQLERRM,1,200);
            RAISE E_ERROR;
      END;

      RSP := 'OK';

      INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE,'PK_VOTACION.DEL_VOTACENSO', PAR_IN, 'RSP: '||RSP);

      COMMIT;

   EXCEPTION
      WHEN E_ERROR THEN

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_VOTACENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

     WHEN OTHERS THEN
        err_num := SQLCODE;
        err_msg := SUBSTR(SQLERRM, 1, 200);

        RSP:= 'ERROR '||TO_CHAR(err_num)||': '||err_msg;

        INSERT INTO LOGS (FALTA, FUNCION, PARIN, PAROUT)
           VALUES(SYSDATE, 'PK_VOTACION.DEL_VOTACENSO', PAR_IN, 'RSP: '||RSP);

        COMMIT;

  END DEL_VOTACENSO;

END PK_VOTACION;
/
spool off;